# Common definitions and functions.
#
# Copyright (C) 2020, 2022 Paul Boddie <paul@boddie.org.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA

IDL_PROG			= $(L4DIR)/idl4re/idl

# Functions to generate filenames for individual interfaces.

export_includes			= $(patsubst %,$(IDL_EXPORT_DIR)/%,$(1))

interfaces_to_idl 		= $(patsubst %,$(IDL_DIR)/%.idl,$(1))

interfaces_to_client_c		= \
				$(patsubst %,%_client.c,$(1)) \

interfaces_to_client_c_h	= \
				$(patsubst %,%_client.c,$(1)) \
				$(patsubst %,%_client.h,$(1)) \

interfaces_to_client_cc		= \
				$(patsubst %,%_client.cc,$(1)) \

interfaces_to_client_cc_h	= \
				$(patsubst %,%_client.cc,$(1)) \
				$(patsubst %,%_client.h,$(1)) \

interfaces_to_interface_h 	= \
				$(patsubst %,%_interface.h,$(1)) \

interfaces_to_interfaces_h 	= \
				$(patsubst %,%_interface.h,$(1)) \
				$(patsubst %,%_interfaces.h,$(1)) \

_interfaces_to_server_c		= \
				$(patsubst %,%_server.c,$(1)) \

_interfaces_to_server_c_h	= \
				$(patsubst %,%_server.c,$(1)) \
				$(patsubst %,%_server.h,$(1)) \

_interfaces_to_server_cc	= \
				$(patsubst %,%_server.cc,$(1)) \

_interfaces_to_server_cc_h	= \
				$(patsubst %,%_server.cc,$(1)) \
				$(patsubst %,%_server.h,$(1)) \

# For each interface file basename, obtain basenames of all other files needed
# to complete the interfaces in the given file. The resulting list is sorted and
# without duplicate words. This function is used to support the indication of
# compound interfaces, providing the constituent interfaces.

needed_interfaces = $(sort $(foreach BASENAME,$(call interfaces_to_idl,$(1)),$(shell $(IDL_PROG) -f $(BASENAME))))

# File generation for all needed interfaces.

interfaces_to_server_c		= $(call _interfaces_to_server_c,$(call needed_interfaces,$(1)))
interfaces_to_server_c_h	= $(call _interfaces_to_server_c_h,$(call needed_interfaces,$(1)))
interfaces_to_server_cc		= $(call _interfaces_to_server_cc,$(call needed_interfaces,$(1)))
interfaces_to_server_cc_h	= $(call _interfaces_to_server_cc_h,$(call needed_interfaces,$(1)))

# Generation of exported interface header file locations.

exported_interfaces_h = $(foreach INTERFACE,$(call interfaces_to_interface_h,$(1)),$(call export_includes,$(INTERFACE)))
